cdef int GI_NOOP         = 1 << 0
cdef int GI_IGNORE       = 1 << 1
cdef int GI_NEEDS_UPDATE = 1 << 2
cdef int GI_GROUP        = 1 << 3
cdef int GI_CONTEXT_MOD  = 1 << 4
cdef int GI_VERTEX_DATA  = 1 << 5
cdef int GI_COMPILER	 = 1 << 6
cdef int GI_NO_APPLY_ONCE = 1 << 7
cdef int GI_NO_REMOVE    = 1 << 8

