'''
The use of id in KV
===================

This small example shows how to refer from one widget
to another within KV.
'''

import kivy
kivy.require('1.8.0')

from kivy.app import App


class TestApp(App):
    pass


if __name__ == '__main__':
    TestApp().run()
